import {
  APPENDING_POSITIONS,
  ExtraPosition,
  FLOATING_BOTTOM,
  FLOATING_LEFT,
  FLOATING_POSITIONS,
  FLOATING_RIGHT,
  FLOATING_TOP,
  PREPENDING_POSITIONS
} from "./chunk-SK2NESLO.mjs";

// src/index.ts
import { createRequire } from "module";
import { createSyncFn } from "synckit";
function getThemeName(theme) {
  if (typeof theme === "string")
    return theme;
  return theme.name;
}
var HIGHLIGHT_RE = /{([\d,-]+)}/;
var BACKGROUND_STYLE_RE = /^<pre[^>]*style="([^"]*)"[^>]*>/;
var DARK_CLASS = "shiki-dark";
var LIGHT_CLASS = "shiki-light";
var FLOAT_CLASS = "shiki-float";
var FLOAT_POS_CLASS_MAPPING = {
  "shiki-float-top": FLOATING_TOP,
  "shiki-float-bottom": FLOATING_BOTTOM,
  "shiki-float-right": FLOATING_RIGHT,
  "shiki-float-left": FLOATING_LEFT
};
var FLOAT_POS_CLASS_ITER = Object.entries(FLOAT_POS_CLASS_MAPPING);
function resolveOptions(options) {
  const themes = [];
  let darkModeThemes;
  if (!options.theme) {
    themes.push("nord");
  } else if (typeof options.theme === "string") {
    themes.push(options.theme);
  } else {
    if ("dark" in options.theme || "light" in options.theme) {
      darkModeThemes = options.theme;
      themes.push(darkModeThemes.dark);
      themes.push(darkModeThemes.light);
    } else {
      themes.push(options.theme);
    }
  }
  return {
    ...options,
    themes,
    darkModeThemes: darkModeThemes ? {
      dark: getThemeName(darkModeThemes.dark),
      light: getThemeName(darkModeThemes.light)
    } : void 0,
    extra: options.extra || []
  };
}
var attrsToLines = (attrs) => {
  const result = [];
  if (!attrs.trim())
    return [];
  attrs.split(",").map((v) => v.split("-").map((v2) => parseInt(v2, 10))).forEach(([start, end]) => {
    if (start && end) {
      result.push(
        ...Array.from({ length: end - start + 1 }, (_, i) => start + i)
      );
    } else {
      result.push(start);
    }
  });
  return result.map((v) => ({
    line: v,
    classes: ["highlighted"]
  }));
};
var processExtra = (extra, attrs, lang, code) => {
  const result = [];
  if (extra === void 0)
    return result;
  extra.forEach((processor) => {
    const matched = processor.attrRe ? processor.attrRe.exec(attrs) : null;
    result.push({
      light: processor.light(matched, lang, code),
      dark: processor.dark === null ? null : (processor.dark || processor.light)(matched, lang, code),
      position: processor.position
    });
  });
  return result;
};
var getStyleContent = (tag, regex) => {
  const match = regex.exec(tag);
  let style_content = "";
  if (match)
    style_content = match[1];
  return style_content;
};
var appendStyle = (intel, style) => {
  if (!intel)
    return;
  if (!style.endsWith(";"))
    style += ";";
  if (intel.attrs.style)
    intel.attrs.style += `;${style}`;
  else
    intel.attrs.style = style;
};
var appendClass = (intel, className) => {
  if (!intel)
    return;
  if (intel.attrs.class)
    intel.attrs.class += ` ${className}`;
  else
    intel.attrs.class = className;
};
var h = (intel) => {
  if (!intel)
    return "";
  const headPartal = `<${intel.tag}${Object.entries(intel.attrs).reduce((prev, now) => {
    return `${prev} ${now[0]}="${now[1]}"`;
  }, "")}`;
  if (intel.content) {
    if (typeof intel.content === "string")
      return `${headPartal}>${intel.content}</${intel.tag}>`;
    else if (Array.isArray(intel.content))
      return `${headPartal}>${intel.content.reduce((prev, now) => prev + h(now), "")}</${intel.tag}>`;
    else
      return `${headPartal}>${h(intel.content)}</${intel.tag}>`;
  } else {
    return `${headPartal} />`;
  }
};
var wrapFinalContainer = (light, dark = void 0, processedExtra = void 0) => {
  const extraContentsOnPosition = Object.keys(ExtraPosition).map(() => "");
  dark = dark || "";
  processedExtra = processedExtra || [];
  const light_style_content = getStyleContent(light, BACKGROUND_STYLE_RE);
  const dark_style_content = getStyleContent(dark, BACKGROUND_STYLE_RE);
  processedExtra.forEach((extra) => {
    if (extra.dark !== null) {
      appendClass(extra.light, LIGHT_CLASS);
      appendStyle(extra.light, light_style_content);
      appendClass(extra.dark, DARK_CLASS);
      appendStyle(extra.dark, dark_style_content);
    } else {
      extra.dark = void 0;
    }
    if (FLOATING_POSITIONS.includes(extra.position)) {
      appendClass(extra.light, FLOAT_CLASS);
      appendClass(extra.dark, FLOAT_CLASS);
    }
    for (const [posClass, posCheckList] of FLOAT_POS_CLASS_ITER) {
      if (posCheckList.includes(extra.position)) {
        appendClass(extra.light, posClass);
        appendClass(extra.dark, posClass);
      }
    }
    const lightExtra = h(extra.light);
    const darkExtra = h(extra.dark);
    extraContentsOnPosition[extra.position] += lightExtra;
    extraContentsOnPosition[extra.position] += darkExtra;
  });
  const before = PREPENDING_POSITIONS.map((i) => extraContentsOnPosition[i]).reduce((prev, exc) => prev + exc, "");
  const after = APPENDING_POSITIONS.map((i) => extraContentsOnPosition[i]).reduce((prev, exc) => prev + exc, "");
  return `<div class="shiki-container" style="position: relative;">${before}${light}${dark}${after}</div>`;
};
var MarkdownItShiki = (markdownit, options = {}) => {
  const _highlighter = options.highlighter;
  const {
    langs,
    themes,
    darkModeThemes,
    highlightLines,
    extra
  } = resolveOptions(options);
  let syncRun;
  if (!_highlighter) {
    const require2 = createRequire(import.meta.url);
    syncRun = createSyncFn(require2.resolve("./worker"));
    syncRun("getHighlighter", { langs, themes });
  }
  const highlightCode = (code, lang, theme, lineOptions) => {
    if (_highlighter)
      return _highlighter.codeToHtml(code, { lang: lang || "text", theme, lineOptions });
    return syncRun("codeToHtml", {
      code,
      theme,
      lang: lang || "text",
      lineOptions
    });
  };
  markdownit.options.highlight = (code, lang, attrs) => {
    let lineOptions;
    if (highlightLines) {
      const match = HIGHLIGHT_RE.exec(attrs);
      if (match)
        lineOptions = attrsToLines(match[1]);
    }
    const processedExtra = processExtra(extra, attrs, lang, code);
    if (darkModeThemes) {
      const dark = highlightCode(code, lang, darkModeThemes.dark, lineOptions).replace('<pre class="shiki', `<pre class="shiki ${DARK_CLASS}`);
      const light = highlightCode(code, lang || "text", darkModeThemes.light, lineOptions).replace('<pre class="shiki', `<pre class="shiki ${LIGHT_CLASS}`);
      return wrapFinalContainer(light, dark, processedExtra);
    } else {
      return wrapFinalContainer(
        highlightCode(
          code,
          lang || "text",
          void 0,
          lineOptions
        ),
        void 0,
        processedExtra
      );
    }
  };
};
var src_default = MarkdownItShiki;
export {
  ExtraPosition,
  src_default as default,
  h,
  resolveOptions
};
